@@ -1,7 +1,7 @@
 import re
 import json
 import inspect
-from funcy import memoize, compose, wraps, any, any_fn, select_values, lmapcat
+from funcy import memoize, compose, wraps, any, any_fn, select_values, mapcat
 
 from django.db import models
 from django.http import HttpRequest
@@ -9,10 +9,6 @@ from django.http import HttpRequest
 from .conf import model_profile
 
 
-def get_table_model(model):
-    return next((b for b in model.__mro__ if issubclass(b, models.Model) and b is not models.Model
-                 and not b._meta.proxy and not b._meta.abstract), None)
-
 def model_family(model):
     """
     The family is models sharing a database table, events on one should affect each other.
@@ -20,14 +16,19 @@ def model_family(model):
     We simply collect a list of all proxy models, including subclasess, superclasses and siblings.
     Two descendants of an abstract model are not family - they cannot affect each other.
     """
+    if model._meta.abstract:  # No table - no family
+        return set()
+
+    @memoize
     def class_tree(cls):
-        return [cls] + lmapcat(class_tree, cls.__subclasses__())
-
-    # NOTE: we also list multitable submodels here, we just don't care.
-    #       Cacheops doesn't support them anyway.
-    table_model = get_table_model(model)
-    return class_tree(table_model) if table_model else []
+        # NOTE: we also list multitable submodels here, we just don't care.
+        #       Cacheops doesn't support them anyway.
+        return {cls} | set(mapcat(class_tree, cls.__subclasses__()))
 
+    table_bases = {b for b in model.__mro__ if issubclass(b, models.Model) and b is not models.Model
+                   and not b._meta.proxy and not b._meta.abstract}
+    family = set(mapcat(class_tree, table_bases))
+    return {cls for cls in family if not cls._meta.abstract}
 
 @memoize
 def family_has_profile(cls):
